package strsets; 

/**
 * Insieme di stringhe modificabile implementato con un array
 * Estende StrSet
 */
public class ArrayStrSet implements ModifiableStrSet {
    static final int initCapacity = 2;      // capacita' iniziale di default
    static final double EXPFACTOR = 1.5;    // fattore di espansione usato per
                                            // calcolare la nuova dimensione
                                            // dell'array
    String[] strArray;    // array che mantiene l'insieme di stringhe
    int strCount;         // numero di stringhe dell'insieme
    
    /**
     * Costruisce un insieme vuoto con un array di capacita' iniziale
     * pari al valore di default
     * 
     * @return                  la nuova istanza
     */
    public ArrayStrSet() {
        this(initCapacity);
    }

    /**
     * Costruisce un insieme vuoto con un array di capacita' iniziale data
     * 
     * @param initCapcacity     La capacita' iniziale
     * @return                  la nuova istanza
     */
    public ArrayStrSet(int initCapacity) {
        strArray = new String[initCapacity];
        strCount = 0;
    }
    
    /**
     * Costruisce un insieme copiando le stringhe da un array
     * 
     * @param sA    array di strighe
     * @return      l'insieme contenente una copia delle stringhe
     */
    public ArrayStrSet(String[] sA) {
        strArray = sA.clone();
        strCount = strArray.length;
    }
    
    /**
     * Metodo ausiliario che cerca una stringa nell'array
     * 
     * @param s     la stringa
     * @return      l'indice della stringa, se presente nell'array, o -1 altrimenti
     */
    private int find(String s) {
        for (int i = 0 ; i < strCount ; i++)
            if (strArray[i].equals(s))
                return i;
        return -1;
    }
    
    /**
     * Aggiunge una stringa all'insieme (se non gia' presente)
     * Se la stringa e' gia' presente, lascia l'insieme immutato.
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param s     la stringa
     * @return      true se la stringa non e' contenuta ed e' stata aggiunta
     *              correttamente, false altrimenti
     */
    public boolean add(String s) {
        if (find(s) == -1) {    // non presente
            if (strArray.length == strCount) {  // se l'array e' pieno...
                int newSize = (int) Math.ceil(EXPFACTOR*(strCount + 1));
                                                // nuova capacita'
                String[] newA = new String[newSize];                // crea copia
                System.arraycopy(strArray, 0, newA, 0, strCount);   // con nuova
                strArray = newA;                                    // capacita'
            }
            strArray[strCount++] = s;   // inserisco elemento in coda all'array
            return true;
        } else
            return false;
    }

    /**
     * Rimunove una stringa dall'insieme (se presente)
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param s     la stringa
     * @return      true se la stringa e' contenuta ed e' stata rimossa
     *              correttamente, false altrimenti
     */
    public boolean remove(String s) {
        int k = find(s);
        if (k >= 0) {   // presente

            strArray[k] = strArray[strCount - 1];   // rimuove la stringa trovata
                                                    // rimpiazzandola con l'ultima
                                                    // stringa nell'array
            strCount--;                             // nuovo num elementi
            return true;
        } else
            return false;
    }

    /**
     * Verifica se una stringa e' contenuta nell'insieme
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @param s     la stringa
     * @return      true se la stringa e' contenuta, false altrimenti
     */
    public boolean contains(String s) {
        return (find(s) >= 0);
    }

    /**
     * Dimensione dell'insieme
     * (implementazione di un metodo dell'interfaccia)
     * 
     * @return      il numero di stringhe nell'insieme
     */
    public int size() {
        return strCount;
    }
}